# 负载均衡（Load Balancing）

在集群负载均衡时，Dubbo 提供了多种均衡策略，缺省为 `random` 随机调用。当然你也可以自行扩展负载均衡策略。



## 负载均衡策略
### Random LoadBalance

随机，按权重设置随机概率。在一个截面上碰撞的概率高，但调用量越大分布越均匀，而且按概率使用权重后也比较均匀，有利于动态调整提供者权重。实例：

```xml
<dubbo:reference id="helloService" loadbalance="random"
                 interface="com.huangx.dubbo.service.HelloService" />
```



### RoundRobin LoadBalance

轮循，按公约后的权重设置轮循比率。存在慢的提供者累积请求的问题，比如：第二台机器很慢，但没挂，当请求调到第二台时就卡在那，久而久之，所有请求都卡在调到第二台上。实例：

```XML
<dubbo:reference id="helloService" loadbalance="roundrobin"
                 interface="com.huangx.dubbo.service.HelloService" />
```



### LeastActive LoadBalance

最少活跃调用数，相同活跃数的随机，活跃数指调用前后计数差。使慢的提供者收到更少请求，因为越慢的提供者的调用前后计数差会越大。实例：

```XML
<dubbo:reference id="helloService" loadbalance="leastactive" timeout="5000"
                 interface="com.huangx.dubbo.service.HelloService" />
```



### ConsistentHash LoadBalance

一致性 Hash，相同参数的请求总是发到同一提供者。当某一台提供者挂时，原本发往该提供者的请求，基于虚拟节点，平摊到其它提供者，不会引起剧烈变动。

算法参见：http://en.wikipedia.org/wiki/Consistent_hashing

缺省只对第一个参数 Hash，如果要修改，请配置 ：

```XML
<!-- 下面将使用服务的第一个和第二个参数作为hash运算key -->
<dubbo:parameter key="hash.arguments" value="0,1" />
```

缺省用 160 份虚拟节点，如果要修改，请配置 ：

```XML
<!-- 设置虚拟节点的数为320个 -->
<dubbo:parameter key="hash.nodes" value="320" />
```



实例：

```XML
<dubbo:reference id="helloExtService" loadbalance="consistenthash" timeout="5000"
                 interface="com.huangx.dubbo.service.HelloExtService" >
    <!-- 将第二个参数作为hash运算的key -->
    <!-- 0,1 将第一个和第二个参数作为hash运算的key -->
    <dubbo:parameter key="hash.arguments" value="2" />
    <dubbo:parameter key="hash.nodes" value="4" />
</dubbo:reference>
```



> 注意：更多关于负载均衡的算法实现请查看Dubbo源代码。









